## Example 5

As mentioned previously, `app` is a handler function that takes `request` and `response` arguments. A simplified version of that handler would look like this:

```javascript
function handler (request, response) {
  var requestPocket = app.pocket();
  requestPocket.value('request', request);
  requestPocket.value('response', response);
  requestPocket.get('responder').then(requestPocket.run) // error handling elided
}
```

First the handler creates a child-pocket from `app`, then adds the `request` and `response` objects, finally it gets the `'responder'` for the pocket and runs it using `pocket.run`.

This `'responder'` is the only missing dependency, so let's define it:

```javascript
app.value('responder', function () {
  return function (response, config) {
    response.end(JSON.stringify(config));
  }
});
```

Because the responder will be run with `pocket.run`, it can depend on a `config` value we haven't defined yet. Let's add that to the app-wide pocket:

```javascript
var fs = require('lie-fs');
app.value('config', loadConfig);

function loadConfig () {
  return fs.readFile(__dirname + '/config.json').then(JSON.parse);
}
```

In a larger application you may want to define `loadConfig` in a separate module that you can test in isolation.


## Example 5 - Augmenting defaults

You can augment the default implementations of `responder` and `result` by overriding them with a function that depends on `defaultResponder` and `defaultResult` respectively. This is useful for things you want to run globally like request logging:

```javascript
var app = createHandler();
app.value('responder', function () {
  return function (defaultResponder, request, response) {
    var start = new Date();
    var method = request.method;
    var url = request.url;
    response.on('finish', function  () {
      console.log(method, url, (new Date()) - start, 'ms');
    });
    return this.run(defaultResponder);
  }
});
```


